@@ -18,11 +18,19 @@
 """
 
 from abc import ABCMeta, abstractmethod
-from btchipException import *
-from ledgerWrapper import wrapCommandAPDU, unwrapResponseAPDU
+from .btchipException import *
+from .ledgerWrapper import wrapCommandAPDU, unwrapResponseAPDU
 from binascii import hexlify
-import hid
 import time
+import os
+import struct
+import socket
+
+try:
+	import hid
+	HID = True
+except ImportError:
+	HID = False
 
 try:
 	from smartcard.Exceptions import NoCardException
@@ -64,15 +72,15 @@ class HIDDongleHIDAPI(Dongle, DongleWait):
 
 	def exchange(self, apdu, timeout=20000):
 		if self.debug:
-			print "=> %s" % hexlify(apdu)
+			print("=> %s" % hexlify(apdu))
 		if self.ledger:
 			apdu = wrapCommandAPDU(0x0101, apdu, 64)		
 		padSize = len(apdu) % 64
 		tmp = apdu
-		if padSize <> 0:
+		if padSize != 0:
 			tmp.extend([0] * (64 - padSize))
 		offset = 0
-		while(offset <> len(tmp)):
+		while(offset != len(tmp)):
 			data = tmp[offset:offset + 64]
 			data = bytearray([0]) + data
 			self.device.write(data)
@@ -114,8 +122,8 @@ class HIDDongleHIDAPI(Dongle, DongleWait):
 		sw = (result[swOffset] << 8) + result[swOffset + 1]
 		response = result[dataStart : dataLength + dataStart]
 		if self.debug:
-			print "<= %s%.2x" % (hexlify(response), sw)
-		if sw <> 0x9000:
+			print("<= %s%.2x" % (hexlify(response), sw))
+		if sw != 0x9000:
 			raise BTChipException("Invalid status %04x" % sw, sw)
 		return response
 
@@ -134,7 +142,7 @@ class HIDDongleHIDAPI(Dongle, DongleWait):
 		if self.opened:
 			try:
 				self.device.close()
-			except:
+			except Exception:
 				pass
 		self.opened = False
 
@@ -148,12 +156,12 @@ class DongleSmartcard(Dongle):
 
 	def exchange(self, apdu, timeout=20000):
 		if self.debug:
-			print "=> %s" % hexlify(apdu)
+			print("=> %s" % hexlify(apdu))
 		response, sw1, sw2 = self.device.transmit(toBytes(hexlify(apdu)))
 		sw = (sw1 << 8) | sw2
 		if self.debug:
-			print "<= %s%.2x" % (toHexString(response).replace(" ", ""), sw)
-		if sw <> 0x9000:
+			print("<= %s%.2x" % (toHexString(response).replace(" ", ""), sw))
+		if sw != 0x9000:
 			raise BTChipException("Invalid status %04x" % sw, sw)
 		return bytearray(response)
 
@@ -161,33 +169,68 @@ class DongleSmartcard(Dongle):
 		if self.opened:
 			try:
 				self.device.disconnect()
-			except:
+			except Exception:
 				pass
 		self.opened = False
 
+class DongleServer(Dongle):
+
+	def __init__(self, server, port, debug=False):
+		self.server = server
+		self.port = port
+		self.debug = debug
+		self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+		try:
+			self.socket.connect((self.server, self.port))
+		except Exception:
+			raise BTChipException("Proxy connection failed")
+
+	def exchange(self, apdu, timeout=20000):
+		if self.debug:
+			print("=> %s" % hexlify(apdu))		
+		self.socket.send(struct.pack(">I", len(apdu)))
+		self.socket.send(apdu)
+		size = struct.unpack(">I", self.socket.recv(4))[0]
+		response = self.socket.recv(size)
+		sw = struct.unpack(">H", self.socket.recv(2))[0]
+		if self.debug:
+			print("<= %s%.2x" % (hexlify(response), sw))
+		if sw != 0x9000:
+			raise BTChipException("Invalid status %04x" % sw, sw)
+		return bytearray(response)
+
+	def close(self):
+		try:
+			self.socket.close()
+		except Exception:
+			pass
+
 def getDongle(debug=False):
 	dev = None
 	hidDevicePath = None
-	ledger = False
-	for hidDevice in hid.enumerate(0, 0):
-		if hidDevice['vendor_id'] == 0x2581 and hidDevice['product_id'] == 0x2b7c:
-			hidDevicePath = hidDevice['path']
-		if hidDevice['vendor_id'] == 0x2581 and hidDevice['product_id'] == 0x3b7c:
-			hidDevicePath = hidDevice['path']			
-			ledger = True
-		if hidDevice['vendor_id'] == 0x2581 and hidDevice['product_id'] == 0x4b7c:
-			hidDevicePath = hidDevice['path']
-			ledger = True
-		if hidDevice['vendor_id'] == 0x2c97 and hidDevice['product_id'] == 0x0000:
-			hidDevicePath = hidDevice['path']
-			ledger = True
-		if hidDevice['vendor_id'] == 0x2581 and hidDevice['product_id'] == 0x1807:
-			hidDevicePath = hidDevice['path']
+	ledger = False	
+	if HID:
+		for hidDevice in hid.enumerate(0, 0):
+			if hidDevice['vendor_id'] == 0x2581 and hidDevice['product_id'] == 0x2b7c:
+				hidDevicePath = hidDevice['path']
+			if hidDevice['vendor_id'] == 0x2581 and hidDevice['product_id'] == 0x3b7c:
+				hidDevicePath = hidDevice['path']			
+				ledger = True
+			if hidDevice['vendor_id'] == 0x2581 and hidDevice['product_id'] == 0x4b7c:
+				hidDevicePath = hidDevice['path']
+				ledger = True
+			if hidDevice['vendor_id'] == 0x2c97:
+				if ('interface_number' in hidDevice and hidDevice['interface_number'] == 0) or ('usage_page' in hidDevice and hidDevice['usage_page'] == 0xffa0):
+					hidDevicePath = hidDevice['path']
+					ledger = True
+			if hidDevice['vendor_id'] == 0x2581 and hidDevice['product_id'] == 0x1807:
+				hidDevicePath = hidDevice['path']
 	if hidDevicePath is not None:
 		dev = hid.device()
 		dev.open_path(hidDevicePath)
 		dev.set_nonblocking(True)
 		return HIDDongleHIDAPI(dev, ledger, debug)
+
 	if SCARD:
 		connection = None
 		for reader in readers():
@@ -201,10 +244,11 @@ def getDongle(debug=False):
 				else:
 					connection.disconnect()
 					connection = None
-			except:
+			except Exception:
 				connection = None
 				pass
 		if connection is not None:
 			return DongleSmartcard(connection, debug)
+	if (os.getenv("LEDGER_PROXY_ADDRESS") is not None) and (os.getenv("LEDGER_PROXY_PORT") is not None):
+		return DongleServer(os.getenv("LEDGER_PROXY_ADDRESS"), int(os.getenv("LEDGER_PROXY_PORT")), debug)
 	raise BTChipException("No dongle found")
-
